@@ -1,12 +1,15 @@
+"""Package of domain model object classes."""
+
 from dm.ioc import *
 from dm.dom.meta import *
 from dm.exceptions import *
 import dm.times
 from dm.util.datastructure import MultiValueDict
+from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
 
 debug = RequiredFeature('Debug')
-domdebug = False
-
+moddebug = False
+    
 class DomainBase(object):
     """
     'Layer Supertype (475)'  [Fowler, 2003]
@@ -17,8 +20,12 @@ class DomainBase(object):
     registry         = RequiredFeature('DomainRegistry')
     pluginController = RequiredFeature('PluginController')
 
+    def resolvePathPart(self, pathPart):
+        return getattr(self, pathPart)
 
-class AbstractList(DomainBase):
+
+# Todo: Rename as AbstractRegister - this class is NOT a list. :-)
+class AbstractList(DomainBase): 
     """
     Supertype for domain model registers.
     Follows 'AbstractList' in 'Iterator (257)' [GoF].
@@ -26,38 +33,34 @@ class AbstractList(DomainBase):
 
     database = RequiredFeature('DatabaseFacade')
 
-    def __init__(self, 
-        owner=None, ownerName=None, owner2=None, 
-        ownerName2=None, isCached=False, **kwds
-    ):
+    def __init__(self, owner=None, ownerName=None, owner2=None,
+            ownerName2=None, filter={}, isCached=False, **kwds):
         self.owner = owner
         self.ownerName = ownerName
         self.owner2 = owner2
         self.ownerName2 = ownerName2
+        self.filter = filter
         self.isCached = isCached
         if self.isCached:
             self.cache = dict()
         else:
             self.cache = None
 
-    def __contains__(self, key):
-        "Supports: if key in register:"
-        return self.has_key(key)
-
-    def __getitem__(self, key):
-        "Supports: item = register[key]"
-        item = self.find(key)
-        return item
-
     def get(self, key, default=None):
-        "Implements: register[key] if key in register, else default (None)."
         try:
             return self.__getitem__(key)
         except:
             return default
                 
+    def __getitem__(self, key):
+        item = self.find(key)
+        return item
+
+    # NB: Use item's key to test containment of item in register.
+    def __contains__(self, key):
+        return self.has_key(key)
+
     def has_key(self, key):
-        "Tests existence of register entity with key."
         if self.isCached:
             if key in self.cache:
                 return True
@@ -72,15 +75,15 @@ class AbstractList(DomainBase):
         "Returns keyed object."
         if self.isCached:
             if key in self.cache:
-                if domdebug and debug:
+                if moddebug and debug:
                     message = "Cache hit for key: '%s'" % key
                     self.log.debug(message)
                 return self.readCache(key)
             else:
-                if domdebug and debug:
+                if moddebug and debug:
                     message = "Cache miss for key: '%s'" % key
                     self.log.debug(message)
-        if domdebug and debug:
+        if moddebug and debug:
             message = "Retrieving from %s record for key: '%s'" % (self, key)
             if message == 'WeekEarmarkTemplate':
                 raise Exception, message
@@ -93,6 +96,7 @@ class AbstractList(DomainBase):
 
     def beginTransaction(self):
         "Begins a transaction."
+        # Commit, commit, commit, rollback, begin, commit, commit.
         return self.database.beginTransaction()
         
 
@@ -120,10 +124,15 @@ class DomainObjectRegister(AbstractList):
 
     isStateful = False
 
-    def __init__(self, typeName='', keyName='id', **kwds):
+    def __init__(self, typeName='', keyName='id', metaAttr=None, **kwds):
         super(DomainObjectRegister, self).__init__(**kwds)
-        self.typeName = typeName
+        # Recorded domain class name.
+        self.typeName = typeName  
+        # Attribute used to key register.
         self.keyName = keyName
+        # Meta object for supported model attribute.
+        self.metaAttr = metaAttr
+        # Attributes used in basic search.
         if 'searchAttributeNames' in kwds:
             self.searchAttributeNames = kwds['searchAttributeNames']
         else:
@@ -141,6 +150,17 @@ class DomainObjectRegister(AbstractList):
             className, typeName, keyName
         )
 
+    def resolvePathPart(self, pathPart):
+        # Make register key from path part.
+        keyMeta = self.getKeyMeta()
+        if keyMeta == None:
+            keyValue = pathPart
+        elif keyMeta.isDomainObjectRef:
+            keyValue = keyMeta.getObjectByKey(pathPart)
+        else:
+            keyValue = pathPart
+        return self[keyValue]
+
     def retrieveItem(self, key):
         "Attempts to retrieve existing domain object from database."
         item = None
@@ -169,15 +189,16 @@ class DomainObjectRegister(AbstractList):
     def __len__(self):
         "Supports: len(registry)"
         return self.count()
-        #return self.database.countRecords(self.typeName)
 
     def __iter__(self, **kwds):
-        "Returns iterator for registered items."
+        "Returns iterator for registered items (not keys!)."
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         results = self.database.listRecords(self.typeName, **kwds)
-        iter = RegisterIterator(results=results, register=self)
-        return iter
+        iterator = RegisterIterator(results=results, register=self)
+        objectList = [i for i in iterator]
+        self.sortDomainObjects(objectList)
+        return iter(objectList)
  
     def __delitem__(self, key):
         "Supports: del registry[key]"
@@ -191,7 +212,7 @@ class DomainObjectRegister(AbstractList):
         return self.database.countRecords(self.typeName, **kwds)
     
     def keys(self, **kwds):
-        "Returns list of keys for registry."
+        "Returns list of keys for register."
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         keys = []
@@ -201,7 +222,11 @@ class DomainObjectRegister(AbstractList):
             key = getattr(record.getDomainObject(), keyName)
             keys.append(key)
         return keys
-    
+
+    def values(self, **kwds):
+        "Returns list of records of register."
+        self.getObjectList()
+
     def startsWith(self, value, attributeName=''):
         kwds = {}
         self.coerceKwds(kwds)
@@ -217,8 +242,8 @@ class DomainObjectRegister(AbstractList):
         self.sortDomainObjects(objectList)
         return objectList
     
-    def search(self, userQuery, attributeNames=None):
-        kwds = {}
+    def search(self, userQuery, attributeNames=None, spaceSplit=True):
+        kwds = {'spaceSplit':spaceSplit}
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         if not attributeNames:
@@ -243,7 +268,7 @@ class DomainObjectRegister(AbstractList):
             if object.isChanged:
                 object.save()
         except KforgeDbError, inst:
-            raise KforgeDomError, inst 
+            raise KforgeDomError, inst
         else:
             return object
 
@@ -280,10 +305,10 @@ class DomainObjectRegister(AbstractList):
         else:
             return None
     
-    def findDomainObjects(self, **kwds):
+    def findDomainObjects(self, __loadedList__=None, **kwds):
         objectList = []
         for record in self.findRecords(**kwds):
-            domainObject = record.getDomainObject()
+            domainObject = record.getDomainObject(__loadedList__)
             objectList.append(domainObject)
         self.sortDomainObjects(objectList)
         return objectList
@@ -344,11 +369,9 @@ class DomainObjectRegister(AbstractList):
             kwds[self.ownerName] = self.owner
         if self.owner2 and self.ownerName2:
             kwds[self.ownerName2] = self.owner2
-# Just speculating that the following lines are useless. - johnbywater
-#        if self.keyName and self.keyName in kwds:
-#            keyValue = kwds[self.keyName]
-#            if issubclass(keyValue.__class__, DomainObject):
-#                kwds[self.keyName] = keyValue
+        for (k, v) in self.filter.items():
+            if k not in kwds:
+                kwds[k] = v
 
     def switchToRecords(self, kwds):
         metaAttributeNames = self.getDomainClassMetaAttributeNames()
@@ -364,7 +387,7 @@ class DomainObjectRegister(AbstractList):
                             msg = "No record for '%s' domainObjectRef: %s" % (
                                 attrName, kwds
                             )
-                            raise Exception, msg
+                            raise KforgeRegistryKeyError, msg
                 
     def initialiseObject(self, object):
         object.initialise(self)
@@ -402,9 +425,25 @@ class DomainObjectRegister(AbstractList):
             keyName = 'id'
         return keyName
 
+    def getKeyMeta(self):
+        "Returns meta attr for register key."
+        keyName = self.getKeyName()
+        metaAttrs = self.getDomainClassMetaAttributeNames()
+        if keyName in metaAttrs:
+            return metaAttrs[keyName]
+        else:
+            return None
+
     def getStartsWithAttributeName(self):
         "Returns name of domain object attribute used in alphabetical index."
-        return self.startsWithAttributeName or self.getKeyName()
+        attrName = self.startsWithAttributeName or self.getKeyName()
+        if attrName == 'id':
+            msg = "Can't match start of 'id' attribute. Try setting the " 
+            msg += "'startsWithAttributeName' on class %s." % (
+                self.getDomainClassMeta().name
+            )
+            raise Exception(msg)
+        return attrName
 
     def getSearchAttributeNames(self):
         "Returns name of attributes used for searching domain objects'."
@@ -414,15 +453,23 @@ class DomainObjectRegister(AbstractList):
         objectList = [domainObject for domainObject in self]
         return objectList
     
+    def getReverseSortedList(self):
+        objectList = self.getSortedList()
+        objectList.reverse()
+        return objectList
+
     def getSortedList(self):
         objectList = self.getObjectList()
-        self.sortDomainObjects(objectList)
+        #self.sortDomainObjects(objectList)
         return objectList
 
     def sortDomainObjects(self, domainObjects):
         if debug:
-            msg = "Sorting domain object list (len %s)." % len(domainObjects)
-            self.log.debug(msg)
+            count = len(domainObjects)
+            if count:
+                className = domainObjects[0].__class__.__name__
+                msg = "Sorting %s %s domain objects." % (count, className)
+                self.log.debug(msg)
         domainObjects.sort(self.cmpDomainObjects)
 
     def cmpDomainObjects(self, x, y):
@@ -466,6 +513,19 @@ class DomainObjectRegister(AbstractList):
         else:
             return None
 
+    def getOptionsRegister(self):
+        keyName = self.getKeyName()
+        classMeta = self.getDomainClassMeta()
+        if keyName != 'id':
+            keyAttrMeta = classMeta.attributeNames[keyName]
+            if keyAttrMeta.isDomainObjectRef:
+                return keyAttrMeta.getAssociatedObjectRegister(None)
+        return self.clone()
+
+    def clone(self):
+        c = self.getDomainClass().createRegister()
+        return c
+
 
 class Register(DomainObjectRegister):
     pass
@@ -483,6 +543,7 @@ class DomainObject(DomainBase):
     paths = RequiredFeature('FileSystem') 
     isConstant = False
     isUnique = True
+    isTemporal = False
     searchAttributeNames = []
     startsWithAttributeName = ''
     isRegistered = False
@@ -490,6 +551,7 @@ class DomainObject(DomainBase):
     registryAttrName = None
     sortOnName = None 
     sortAscending = True
+    nextAscends = True
 
     def __init__(self, **kwds):
         "Initialise base attributes."
@@ -534,12 +596,14 @@ class DomainObject(DomainBase):
                     reprAttrs += ' %s' % attr.createObjectRepr(self)
         return reprAttrs
 
-
     def getSortOnValue(self):
         if self.sortOnName:
-            return getattr(self, self.sortOnName)
+            sortOnValue = getattr(self, self.sortOnName)
         else:
-            return self.getPersonalLabel()
+            sortOnValue = self.getPersonalLabel()
+        if type(sortOnValue) in [str, unicode]:
+            sortOnValue = sortOnValue.lower()
+        return sortOnValue
 
     def asNamedValues(self, excludeName=''):  # todo: excludeNames
         namedValues = [] 
@@ -548,6 +612,7 @@ class DomainObject(DomainBase):
                 if not excludeName == attr.name:
                     namedValue = {}
                     namedValue['name'] = attr.name
+                    namedValue['title'] = attr.calcTitle()
                     namedValue['value'] = attr.createValueRepr(self)
                     namedValue['label'] = attr.createLabelRepr(self)
                     namedValue['domainKeyValue'] = self.getRegisterKeyValue()
@@ -564,6 +629,13 @@ class DomainObject(DomainBase):
                     namedValues.append(namedValue)
         return namedValues
 
+    def asSortableValues(self):
+        values = {} 
+        if self.meta:
+            for attr in self.meta.attributes:
+                values[attr.name] = attr.createSortableRepr(self)
+        return values
+
     def asDictValues(self):
         dictValues = {} 
         if self.meta:
@@ -571,10 +643,15 @@ class DomainObject(DomainBase):
                 dictValues[attr.name] = attr.createValueRepr(self)
         return dictValues
 
-    def asRequestParams(self):
+    def present(self):
+        return self.asDictLabels()
+
+    def asRequestParams(self, attrNames=None):
         requestParams = MultiValueDict()
         for attr in self.meta.attributes:
             attrName = attr.name
+            if (attrNames != None) and (attrName not in attrNames):
+                continue
             attrValueRepr = attr.createValueRepr(self)
             if attr.isList():
                 requestParams.setlist(attrName, attrValueRepr)
@@ -601,7 +678,7 @@ class DomainObject(DomainBase):
             message = "Class '%s' has no meta object." % className
             raise DomainClassRegistrationError(message)
         if len(self.meta.attributesDeferred):
-            message = "Class '%s' has mismatched attributes: %s" % (
+            message = "Class '%s' has mismatched (deferred) attributes: %s" % (
                 className, str(self.meta.attributesDeferred))
             raise DomainClassRegistrationError(message)
 
@@ -618,7 +695,8 @@ class DomainObject(DomainBase):
 
     def saveSilently(self):
         "Update record without raising update event."
-        self.updateLastModifiedTime()
+        if not self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            self.updateLastModifiedTime()
         self.record.saveDomainObject()
 
     def updateLastModifiedTime(self):
@@ -645,10 +723,21 @@ class DomainObject(DomainBase):
         "Destroy record of object in the database."
         self.decacheItem()
         if self.record:
+            self.deleteTemporalObjects()
             self.record.domainObject = None
             self.record.destroySelf()
             self.record = None
 
+    def deleteTemporalObjects(self):
+        if self.meta.isTemporal:
+            register = self.meta.createTemporalCollection(self)
+            [i.delete() for i in register]
+        else:
+            for metaAttr in self.meta.attributes:
+                if metaAttr.isTemporal:
+                    register = metaAttr.createTemporalCollection(self)
+                    [i.delete() for i in register]
+
     def purgeAggregates(self):
         for metaAttr in self.meta.attributes:
             if metaAttr.isAggregation():
@@ -762,7 +851,7 @@ class DomainObject(DomainBase):
             kwds['searchAttributeNames'] = self.searchAttributeNames
         if not('startsWithAttributeName' in kwds and kwds['startsWithAttributeName']):
             kwds['startsWithAttributeName'] = self.startsWithAttributeName
-        if domdebug and debug:
+        if moddebug and debug:
             className = self.registerClass.__name__
             message = "Creating '%s' register with kwds: %s" % (className, kwds)
             self.log.debug(message)
@@ -791,7 +880,7 @@ class DomainObject(DomainBase):
     # todo: rename to getLabel
     def getLabelValue(self):
         # default to register key value
-        return self.getRegisterKeyValue()
+        return str(self.getRegisterKeyValue())
 
     def getPersonalLabel(self):
         # default to basic label
@@ -816,6 +905,12 @@ class DomainObject(DomainBase):
         
     createNoun = classmethod(createNoun)
 
+    def getOptionsRegister(self, attrName):
+        attrValue = getattr(self, attrName)
+        metaAttr = self.meta.attributeNames[attrName]
+        register = metaAttr.getAssociatedObjectRegister(attrValue)
+        return register
+
 
 class SimpleObject(DomainObject):
     pass
@@ -824,5 +919,5 @@ class SimpleObject(DomainObject):
 class SimpleNamedObject(DomainObject):
 
     registerKeyName = 'name'
-    name = String()
+    name = String(isIndexed=True)
 
