@@ -1,22 +1,34 @@
 """
-KForge meta classes describe KForge DomainObject classes.
+Meta classes describe the DomainObject classes.
 """
 
 from dm.ioc import *
 import md5
 import mx.DateTime
 from dm.datetimeconvertor import DateTimeConvertor
+from dm.datetimeconvertor import RDateTimeConvertor
+from dm.datetimeconvertor import RNSDateTimeConvertor
 from dm.datetimeconvertor import DateConvertor
+from dm.datetimeconvertor import RDateConvertor
+from dm.datetimeconvertor import DateOfBirthConvertor
 import Image
 from dm.dictionarywords import IMAGES_DIR_PATH
 from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
 import os
+import inspect
+from dm.exceptions import *
+moddebug = False
+
+
+
+# todo: Add Email attribute meta class.
 
 class NotDefined(object):
     "Internal 'null' value."
     pass
 
 
+# Todo: Somthing about this being used *only* in the db layer (now).
 class MetaBase(object):
     "Domain model meta supertype."
 
@@ -25,9 +37,54 @@ class MetaBase(object):
     logger = RequiredFeature('Logger')
     debug = RequiredFeature('Debug')
 
+
+class BaseDomainMeta(object):
+    "Domain model meta supertype."
+
+    registry = RequiredFeature('DomainRegistry')
+    dictionary = RequiredFeature('SystemDictionary')
+    logger = RequiredFeature('Logger')
+    debug = RequiredFeature('Debug')
+
+    def resolvePathPart(self, pathPart):
+        return pathPart
+
+    def calcTitle(self):
+        if self.title:
+            return self.title
+        else:
+            return self.spaceFromCamel(self.name)
+
+    def spaceFromCamel(self, camel):
+        AtoZ = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+        spaceSep = ''
+        lastChar = ''
+        for char in camel:
+            if lastChar and (char in AtoZ) and not (lastChar in AtoZ):
+            #if lastChar and (char in AtoZ):
+                spaceSep += ' ' + char.lower()
+            else:
+                spaceSep += char
+            lastChar = char
+        return spaceSep
+
+    def setTemporalDomainClass(self, domainClass):
+        self.temporalDomainClass = domainClass
+
+    def createTemporalCollection(self, domainObject):
+        if not domainObject:
+            raise Exception, "Need a domain object! %s" % domainObject
+        register = self.temporalDomainClass.createRegister(
+            ownerName='parent',
+            owner=domainObject,
+            metaAttr=self
+        )
+        return register
+        
     
-class MetaDomainObject(MetaBase):
+class MetaDomainObject(BaseDomainMeta):
     "Models a domain object."
+    # Todo: Rename to DomainObjectMeta?
 
     reservedAttrNames = ('id', 'record', 'registerCache', 'projectUrls', 'paths', 'pluginController', 'metaClass', 'meta', 'registerKeyName', 'registerClass', 'registry') 
 
@@ -39,17 +96,29 @@ class MetaDomainObject(MetaBase):
         self.isUnique = isUnique  # can't create if match exists
         self.isCached = isCached  # hold in memory
         self.title = title        # presentable name
+        #if not self.title:
+        #    self.title = self.spaceFromCamel(self.name)
+        #self.title = self.spaceFromCamel(self.name)
         self.comment = comment    # presentable explanation
         self.isEditable = isEditable
         self.isHidden = isHidden
         if self.isHidden:
             self.isEditable = False
         self.attributesDeferred = {}
+        self.isTemporal = False
+        self.sortOn = None
+
+    def setSortOnAttr(self, sortOnName):
+        if sortOnName and sortOnName != 'id':
+            sortOn = self.attributeNames[sortOnName]
+            super(MetaDomainObject, self).__setattr__('sortOn', sortOn)
 
     def __setattr__(self, attrName, attrVal):
         if issubclass(attrVal.__class__, MetaDomainAttr):
-            if attrName in self.reservedAttrNames:
-                raise Exception, "Domain attribute name '%s' is reserved." % attrName
+            #if attrName in self.reservedAttrNames:
+            if attrName in self.getReservedAttrNames():
+                msg = "Domain attribute name '%s' is reserved." % attrName
+                raise Exception(msg)
             if not attrName in self.attributeNames:
                 attrVal.name = attrName
                 self.attributes.append(attrVal)
@@ -57,9 +126,16 @@ class MetaDomainObject(MetaBase):
         else:
             super(MetaDomainObject, self).__setattr__(attrName, attrVal)
 
+    def getReservedAttrNames(self):
+        from dm.dom.base import DomainObject
+        return [i[0] for i in inspect.getmembers(DomainObject)]
+        #return DomainObject.__dict__.keys()
+
     def createDomainClass(self, baseClass):
         className = self.name
-        classAttrs = {'meta': self}
+        classAttrs = {
+            'meta': self,
+        }
         return self.createClass(className, baseClass, classAttrs)
 
     def createClass(self, name, base, attrs):
@@ -68,8 +144,16 @@ class MetaDomainObject(MetaBase):
     def hasDeferredAttributes(self):
         return len(self.attributesDeferred) != 0
 
+    def getAttr(self, attrName):
+        if not attrName in self.attributeNames:
+            msg = "Attribute '%s' not defined on domain class '%s'." % (
+                attrName, self.name
+            )
+            raise Exception(msg)
+        return self.attributeNames[attrName]
 
-class MetaDomainAttr(MetaBase):
+
+class MetaDomainAttr(BaseDomainMeta):
     "Models a domain object attribute."
 
     isValueRef = False
@@ -77,13 +161,16 @@ class MetaDomainAttr(MetaBase):
     isAssociateList = False
     isImageFile = False
 
-    def __init__(self, typeName='', name='', dbName='', default=NotDefined, title='', comment='', isEditable=True, isHidden=False, isRequired=True, getChoices=None, **kwds):
+    def __init__(self, typeName='', name='', dbName='', default=NotDefined, title='', comment='', isEditable=True, isHidden=False, isRequired=True, getChoices=None, isTemporal=False, isBitemporal=False, isIndexed=False, **kwds):
         self.typeName = typeName
         self.name = name
         self.dbName = dbName or name
         if default != NotDefined: 
             self.default = default
-        self.title = title
+        self.title = title        # presentable name
+        #if not self.title:
+        #    self.title = self.spaceFromCamel(self.name)
+        #self.title = self.spaceFromCamel(self.name)
         self.comment = comment
         self.isEditable = isEditable
         self.isHidden = isHidden
@@ -91,6 +178,14 @@ class MetaDomainAttr(MetaBase):
             self.isEditable = False
         self.isRequired = isRequired
         self.getChoices = getChoices
+        self.isIndexed = isIndexed
+        self.isTemporal = isTemporal
+        if self.isIndexed and self.isTemporal:
+            raise Exception, "Can't be indexed and temporal, at the mo."
+        self.isBitemporal = isBitemporal
+        if self.isBitemporal:
+            self.isTemporal = True
+        self.isBitemporalActual = False
 
     def __repr__(self):
         return "<%s typeName='%s' name='%s'>" % (self.__class__.__name__, self.typeName, self.name)
@@ -98,6 +193,7 @@ class MetaDomainAttr(MetaBase):
     def isValueObject(self):
         return False
 
+    # Todo: Remove; apparently redundant given isAssociateList attribute?.
     def isList(self):
         return False
 
@@ -125,6 +221,9 @@ class MetaDomainAttr(MetaBase):
             valueRepr = "!!VALUE_REPR_ERROR: %s!!" % inst
         return "%s=\'%s\'" % (self.name, valueRepr)
 
+    def createSortableRepr(self, domainObject):
+        return self.createValueRepr(domainObject)
+
     def createValueRepr(self, domainObject):
         value = getattr(domainObject, self.name)
         return str(value)
@@ -179,11 +278,27 @@ class MetaDomainAttr(MetaBase):
         dbValue = domainValue
         return dbValue
 
+    def duplicateTemporal(self):
+        attrMeta = self.duplicateSelf()
+        if self.isBitemporal:
+             attrMeta.isBitemporal = False
+             attrMeta.isBitemporalActual = True
+             attrMeta.isTemporal = True
+        else:
+             attrMeta.isTemporal = False
+        attrMeta.isRequired = self.isRequired
+        return attrMeta
+
+    def duplicateSelf(self):
+        return type(self)()
+
 
 class ValueObjectAttr(MetaDomainAttr):
     "Models a domain object value attribute."
 
     isValueRef = True
+    isBoolean = False
+    isNumber = False
     isDateTime = False
     isBLOB = False
 
@@ -199,8 +314,16 @@ class ValueObjectAttr(MetaDomainAttr):
             return self.default
         return NotDefined  # todo: support 'null' default, or forget distinction
 
+    def createSortableRepr(self, domainObject):
+        value = getattr(domainObject, self.name)
+        return value
+
     def makeValueFromMultiValueDict(self, multiValueDict):
-        return multiValueDict.get(self.name, '')
+        attrValue = multiValueDict.get(self.name, '')
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
 
 
 class String(ValueObjectAttr):
@@ -211,11 +334,27 @@ class String(ValueObjectAttr):
 
 
 class Text(String):
-    "Models a domain object textual attribute."
+    "Models a domain object textual attribute (multi-line)."
 
     def __init__(self, default='', **kwds):
         super(Text, self).__init__(default=default, **kwds)
 
+from markdown import Markdown
+class MarkdownText(Text):
+    "Models a 'markdown' textual attribute."
+
+    # Todo: Figure out how to render separately from read value?
+    #     Ans: Implement this method:
+    #def createLabelRepr(self, domainObject):
+
+    def createLabelRepr(self, domainObject):
+        attrValue = getattr(domainObject, self.name)
+        if attrValue:
+            md = Markdown(safe_mode='escape')
+            return md.convert(attrValue)
+        else:
+            return ''
+
 
 class Url(String):
     "Models a domain object url attribute."
@@ -239,11 +378,15 @@ class Password(String):
     def makeValueFromMultiValueDict(self, multiValueDict):
         passwordText = multiValueDict[self.name]
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
-            return passwordText
+            attrValue = passwordText
         elif passwordText:
-            return self.makeDigest(passwordText)
+            attrValue = self.makeDigest(passwordText)
         else:
-            return ''
+            attrValue = ''
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
 
     def makeDigest(self, clearText):
         return md5.new(clearText).hexdigest()
@@ -264,13 +407,34 @@ class DateTime(ValueObjectAttr):
 
     def makeValueFromMultiValueDict(self, multiValueDict):
         dateTimeString = multiValueDict[self.name]
-        return self.convertor.fromHTML(dateTimeString)
+        attrValue = self.convertor.fromHTML(dateTimeString)
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
 
     def createValueRepr(self, domainObject):
         dateTimeObject = getattr(domainObject, self.name)
         dateTimeString = self.convertor.toHTML(dateTimeObject)
         return dateTimeString
             
+    def createLabelRepr(self, domainObject):
+        dateTimeObject = getattr(domainObject, self.name)
+        dateTimeString = self.convertor.toLabel(dateTimeObject)
+        return dateTimeString
+
+
+class RDateTime(DateTime):
+    "Models a domain object date attribute."
+    
+    convertor = RDateTimeConvertor()
+
+
+class RNSDateTime(DateTime):
+    "Models a domain object date attribute."
+    
+    convertor = RNSDateTimeConvertor()
+
 
 class Date(DateTime):
     "Models a domain object date attribute."
@@ -278,10 +442,23 @@ class Date(DateTime):
     convertor = DateConvertor()
 
 
+class RDate(Date):
+    "Models a domain object date attribute, but with 'reverse' ANSI format."
+    
+    convertor = RDateConvertor()
+
+
+class DateOfBirth(RDate):
+    "Models a date of birth."
+    
+    convertor = DateOfBirthConvertor()
+
+
 class Boolean(ValueObjectAttr):
     "Models a domain object boolean attribute."
 
-    trueStrings = ['true', 't', '1', 'yes', 'y']
+    isBoolean = True
+    trueStrings = ['on', 'true', 't', '1', 'yes', 'y']
 
     def __init__(self, default=False, isRequired=False, **kwds):
         super(Boolean, self).__init__(
@@ -289,26 +466,65 @@ class Boolean(ValueObjectAttr):
         )
 
     def makeValueFromMultiValueDict(self, multiValueDict):
-        boolVal = multiValueDict[self.name] or False
+        boolVal = multiValueDict[self.name]
         if boolVal.__class__ == False.__class__:
-            return boolVal
+            attrValue = boolVal
         elif boolVal.__class__ == ''.__class__:
-            return boolVal.lower() in self.trueStrings
+            attrValue = boolVal.lower() in self.trueStrings
         else:
-            return False
+            attrValue = False
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
 
     def createValueRepr(self, domainObject):
-        return getattr(domainObject, self.name)
+        attrValue = getattr(domainObject, self.name)
+        if attrValue:
+            return 'on'
+        else:
+            return ''
+
+    def createLabelRepr(self, domainObject):
+        attrValue = getattr(domainObject, self.name)
+        if attrValue:
+            return 'On'
+        else:
+            return 'Off'
 
 
 class Integer(ValueObjectAttr):
     "Models a domain object integer attribute."
 
+    isNumber = True
+
     def __init__(self, default=0, **kwds):
         super(Integer, self).__init__(default=default, **kwds)
 
+    def resolvePathPart(self, pathPart):
+        return int(pathPart)
+
     def makeValueFromMultiValueDict(self, multiValueDict):
-        return int(multiValueDict[self.name])
+        integerValue = multiValueDict[self.name]
+        if integerValue:
+            attrValue = int(integerValue)
+        else:
+            if hasattr(self, 'default'):
+                attrValue = self.default
+            else:
+                attrValue = 0
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
+
+    def createValueRepr(self, domainObject):
+        value = getattr(domainObject, self.name)
+        return value
+
+    def createLabelRepr(self, domainObject):
+        value = getattr(domainObject, self.name)
+        return str(value)
 
 
 class BLOB(ValueObjectAttr):
@@ -409,8 +625,15 @@ class DomainObjectAssociation(MetaDomainAttr):
         
     def getObjectByKey(self, key):
         domainRegister = self.getDomainRegister()
+        if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            #if hasattr(domainRegister, 'getAll'):
+            if domainRegister.isStateful:
+                domainRegister = domainRegister.getAll()
         return domainRegister[key]
 
+    def resolvePathPart(self, pathPart):
+        return self.getObjectByKey(pathPart)
+
 
 class DomainObjectRef(DomainObjectAssociation):
     "Associates a domain object with a single aquaintance."
@@ -453,13 +676,20 @@ class DomainObjectRef(DomainObjectAssociation):
             return str(value)
        
     def makeValueFromMultiValueDict(self, multiValueDict):
-        attrValueKey = multiValueDict[self.name]
-        if not attrValueKey:
-            if hasattr(self, 'default'):
-                attrValueKey = self.default
-        if not attrValueKey:
-            return None
-        return self.getObjectByKey(attrValueKey)
+        refValue = multiValueDict[self.name]
+        if refValue:
+            attrValue = self.getObjectByKey(refValue)
+        elif hasattr(self, 'default'):
+            attrValue = self.getObjectByKey(self.default)
+        else:
+            attrValue = None
+        if moddebug and self.debug:
+            msg = "Made %s value: %s" % (self.name, attrValue)
+            self.logger.debug(msg)
+        return attrValue
+
+    def duplicateSelf(self):
+        return type(self)(self.typeName)
 
 
 class Aggregation(object):
@@ -491,6 +721,85 @@ class Composite(Composition, Aggregate):
     pass
 
 
+class TemporalRegisterMixin(object):
+    "Use real register if in real time, otherwise use temporal history."
+
+    # When using real register, call super methods with given params.
+    # To use temporal history, get parent's revision object and pull
+    # keys and values from it's register.
+
+    timepoint = RequiredFeature('Timepoint')
+
+    # Todo: Fix this before using temporal properties on HasMany attrs!
+
+
+    # Methods interrupted.
+
+    def __iter__(self):
+        if self.timepoint.isReset(): 
+            return super(TemporalRegisterMixin, self).__iter__()
+        else:
+            return iter(self.getCurrentValues())
+
+    def keys(self, **kwds):
+        if self.timepoint.isReset(): 
+            return super(TemporalRegisterMixin, self).keys(**kwds)
+        else:
+            return self.getCurrentKeys()
+
+    def has_key(self, key):
+        if self.timepoint.isReset(): 
+            return super(TemporalRegisterMixin, self).has_key(key)
+        else:
+            return key in self.getCurrentKeys()
+
+    def find(self, key):
+        if self.timepoint.isReset(): 
+            return super(TemporalRegisterMixin, self).find(key)
+        else:
+            return self.getCurrentDict()[key]
+
+    def count(self):
+        if self.timepoint.isReset(): 
+            return super(TemporalRegisterMixin, self).count()
+        else:
+            return len(self.getCurrentAssociations())
+
+
+    # Methods introduced.
+
+
+    def isRealTime(self):
+        if self.timepoint.isReset():
+            return True
+        elif not self.owner:
+            raise Exception, "Temporal registers need an owner for versions!"
+        else:
+            return False
+
+    def getCurrentDict(self):
+        associations = self.getCurrentAssociations()
+        currentDict = {}
+        for a in associations:
+            currentDict[a.recordedKey] = a.recordedValue
+        return currentDict
+           
+    def getCurrentKeys(self):
+        associations = self.getCurrentAssociations()
+        return [a.recordedKey for a in associations]
+           
+    def getCurrentValues(self):
+        associations = self.getCurrentAssociations()
+        return [a.recordedValue for a in associations]
+           
+    def getCurrentAssociations(self):
+        currentVersion = self.getCurrentVersion()
+        return getattr(currentVersion, self.metaAttr.name)
+
+    def getCurrentVersion(self):
+        return self.owner.temporalHistory.getCurrent()
+
+
 class AssociateList(DomainObjectAssociation):
     "Models aquaintance with several domain objects."
 
@@ -502,7 +811,8 @@ class AssociateList(DomainObjectAssociation):
         super(AssociateList, self).__init__(typeName=className, isRequired=isRequired, **kwds)
         self.key = key
         self.owner = owner
-    
+   
+    # Todo: Remove; apparently redundant given isAssociateList attribute?.
     def isList(self):
         return True
 
@@ -521,20 +831,30 @@ class AssociateList(DomainObjectAssociation):
             if self.isKeyDomainObject():
                 label = associateObject.getLabelValue()
             else:
-                label = key
+                label = associateObject.getLabelValue()  # Trying to get 
+                # ... iteration label values in concern update view. Was:
+                # label = key 
             choice = (key, label)
             choices.append(choice)
         return choices
 
     def createInitialValue(self, domainObject):
-        className = self.typeName
-        recordClass = self.registry.getDomainClass(className)
+        # Just instantiate a register for this attribute on this object.
+        return self.createRegister(domainObject)
+
+    def createRegister(self, domainObject):
+        recordClass = self.registry.getDomainClass(self.typeName)
         registerClass = recordClass.registerClass
+        if self.isTemporal:
+            className = "Temporal%s" % registerClass.__name__
+            classBases = (TemporalRegisterMixin, registerClass)
+            registerClass = type(className, classBases, {})
         register = registerClass(
             typeName=self.typeName,
             keyName=self.key,
             ownerName=self.owner,
-            owner=domainObject
+            owner=domainObject,
+            metaAttr=self
         )
         return register
 
@@ -582,18 +902,22 @@ class AssociateList(DomainObjectAssociation):
     def createAssociatedNamedValues(self, domainObject):
         associatedNamedValues = []
         associationRegister = self.getAssociationObjectRegister(domainObject)
-        if self.isKeyDomainObject():  # not bomb proof, really
-            for associatedObject in associationRegister.keys():
+        if self.isKeyDomainObject():  # Non-bomb proof implied many-many, fix.
+            associationObjects = associationRegister.getSortedList()
+            for associationObject in associationObjects:
+                associatedObject = associationRegister.getRegisterKey(
+                    associationObject
+                )
                 namedValue = {}
                 keyValue = associatedObject.getRegisterKeyValue() 
                 namedValue['key'] = keyValue
                 labelValue = str(associatedObject.getLabelValue())
                 namedValue['label'] = labelValue
                 namedValue['associatedObject'] = associatedObject
-                namedValue['associationObject'] = associationRegister[associatedObject]
+                namedValue['associationObject'] = associationObject
                 associatedNamedValues.append(namedValue)
         else:
-            for associatedObject in associationRegister:
+            for associatedObject in associationRegister.getSortedList():
                 namedValue = {}
                 keyValue = associatedObject.getRegisterKeyValue() 
                 namedValue['key'] = keyValue
@@ -648,18 +972,26 @@ class AssociateList(DomainObjectAssociation):
             raise Exception(msg)
         
     def getAssociatedObjectRegister(self, domainObject):
-        associateRegister = None
+        register = None
+        #
+        # Note well:
+        #
+        # Implication that if they key is a domain object the focus is on the
+        # keys and the register objects are many-many association objects.
+        #
+        # Todo: Make this explicit somehow, like with isKeyAssociate() rather
+        #       than the implicit isKeyDomainObject inference.
         if self.isKeyDomainObject():
             keyMeta = self.getKeyMetaAttribute()
             associateClassName = keyMeta.typeName
             associateClass = self.registry.getDomainClass(associateClassName)
-            associateRegister = associateClass.createRegister()
+            register = associateClass.createRegister()
         else:
             if domainObject:
-                associateRegister = getattr(domainObject, self.name)
+                register = self.getAssociationObjectRegister(domainObject)
             else:
-                associateRegister = []
-        return associateRegister   
+                register = []
+        return register   
 
     def getAssociationObject(self, domainObject, keyValue):
         register = self.getAssociationObjectRegister(domainObject)
