@@ -4,6 +4,10 @@ import md5
 import random
 import sys
 import mx.DateTime
+from dm.dictionarywords import WEBKIT_NAME
+from dm.dictionarywords import DJANGO_SECRET_KEY
+from dm.dictionarywords import PYLONS_SECRET_KEY
+from dm.exceptions import WebkitError
 
 class Session(SimpleDatedObject):
     "Visitor session."
@@ -22,7 +26,13 @@ class Session(SimpleDatedObject):
 
     def createKey(self):
         "Returns new session key."
-        secretKey = self.dictionary['django.secret_key'] 
+        webkitName = self.dictionary[WEBKIT_NAME]
+        if webkitName == 'django':
+            secretKey = self.dictionary[DJANGO_SECRET_KEY] 
+        elif webkitName == 'pylons':
+            secretKey = self.dictionary[PYLONS_SECRET_KEY] 
+        else:
+            secretKey = 'soso' 
         randomStr = str(random.randint(0, sys.maxint - 1)) + secretKey
         return md5.new(randomStr).hexdigest()
 
