@@ -80,6 +80,7 @@ class CompoundRegister(StatefulRegister):
             register.requiredStateName = ''
         else:
             register.requiredStateName = stateName
+        register.isStateful = False
         return register
         
 
@@ -132,6 +133,7 @@ class SimpleDatedObject(SimpleObject):
 
     dateCreated = DateTime(
         default=dm.times.getUniversalNow,
+        isIndexed=True,
         isEditable=False
     )
     lastModified = DateTime(
