@@ -9,17 +9,18 @@ from dm.strategy import MakeCheckString
 from dm.strategy import MakeCookieString
 from dm.strategy import ValidateCookieString
 from dm.util.datastructure import MultiValueDict
-from dm.util.http import HttpResponse
-from dm.util.http import HttpResponseRedirect
-from dm.util.template import Context
-from dm.util.template import template_loader
+from dm.webkit import HttpResponse
+from dm.webkit import HttpResponseRedirect
+from dm.webkit import Context
+from dm.webkit import template_loader
 import dm.times
 import os
 
-moddebug = False
+moddebug = True
 
 class ControlledAccessView(object):
 
+    responseStatusCode = 200
     accessController = RequiredFeature('AccessController')
     dictionary       = RequiredFeature('SystemDictionary')
     commands         = RequiredFeature('CommandSet')
@@ -33,18 +34,22 @@ class ControlledAccessView(object):
     if AUTH_COOKIE_DOMAIN in dictionary:
         authCookieDomain = dictionary[AUTH_COOKIE_DOMAIN]
     else:
-        # prefix with '.' to match subdomains
+        # Prefix with '.' to match subdomains.
         authCookieDomain = '.' + dictionary[DOMAIN_NAME]
 
-    def __init__(self, request=None, **kwds):
+    def __init__(self, request=None, masterView=None, **kwds):
         self.request = request
+        self.masterView = masterView
         self.response = None
+        self.path = None
         self.session = None
         # todo: clarify meaning of redirect and redirectPath
         self.redirect = ''
         self.redirectPath = ''
         self.returnPath = ''
-        # todo: clean up all this guff
+        if self.isSlaveView(): self.initFromMaster()
+
+        # Todo: Clean up all this guff.
         self._canReadSystem = None
         self._canUpdateSystem = None
         self._canCreatePerson = None
@@ -67,6 +72,13 @@ class ControlledAccessView(object):
         self._canUpdateService = None
         self._canDeleteService = None
 
+    def initFromMaster(self):
+        self.path = self.masterView.path
+        self.session = self.masterView.session
+
+    def isSlaveView(self):
+        return self.masterView != None
+
     def getMethodName(self):
         if self.request.POST:
             return 'POST'
@@ -78,17 +90,18 @@ class ControlledAccessView(object):
             sessionKey = self.makeSessionKeyFromCookieString(cookieString)
         except KforgeSessionCookieValueError:
             self.session = None
-            if self.debug:
-                self.logger.debug(
-                    'Session cookie value error: %s' % cookieString
-                )
+            self.logger.error(
+                'Session cookie value error: %s' % cookieString
+            )
         else:
             self.session = self.findSession(sessionKey)
             if self.session:
                 self.session.updateLastVisited()
                 if self.debug:
                     self.logger.debug(
-                        'Session re-established: %s' % self.session
+                        'Session re-established (%s): %s' % (
+                            self.__class__.__name__, self.session
+                        )
                     )
             else:
                 if self.debug:
@@ -341,12 +354,14 @@ class ControlledAccessView(object):
         return self._canDeleteService
 
     def getResponse(self):
-        return HttpResponse('Sorry, the getResponse() method is not implemented on class %s' % self.__class__)
+        msg = 'The getResponse() method is not implemented on class %s' % (
+            self.__class__
+        )
+        return HttpResponse(msg)
 
 
 class SessionView(ControlledAccessView):
 
-    #templateLoader = TemplateLoader()
     templatePath = None
     majorNavigation = []
     majorNavigationItem = "/"
@@ -355,15 +370,15 @@ class SessionView(ControlledAccessView):
 
     def __init__(self, **kwds):
         super(SessionView, self).__init__(**kwds)
-        self.template     = None
-        self.context      = None
+        self.requestParams = None
+        self.template = None
+        self.context = None
         self.sessionCookieString = ''
         self.person = None
         self.project = None
         self.member = None
         self.service = None
         self.readRequest()
-        self.requestParams = None
 
     def __repr__(self):
         return "<%s redirect='%s' redirectPath='%s'>" % (
@@ -382,6 +397,7 @@ class SessionView(ControlledAccessView):
         ))
 
     def setPathFromRequest(self):
+        if self.path != None: return
         uriPrefix = self.dictionary[URI_PREFIX]
         requestPath = self.request.path 
         if uriPrefix and requestPath.startswith(uriPrefix):
@@ -391,26 +407,32 @@ class SessionView(ControlledAccessView):
 
     def readRequestSession(self):
         "Determines session from a cookie in request."
+        if self.session != None:
+            return
         cookieName = self.authSessionCookieName
         cookieString = self.request.COOKIES.get(cookieName, '')
         if not cookieString:
             if self.debug:
-                self.logger.debug('No authentication session cookie in request.')
+                msg = 'No authentication session cookie in request.'
+                self.logger.debug(msg)
             return
         if cookieString == self.noAuthSessionCookieString:
             if self.debug:
-                self.logger.debug('Unauthenticated session cookie in request.')
+                msg = 'Unauthenticated session cookie in request.'
+                self.logger.debug(msg)
             return
         if self.debug:
-            self.logger.debug('Session cookie in request...')
+            msg = 'Session cookie in request... (%s)' % (
+                self.__class__.__name__
+            )
+            self.logger.debug(msg)
         self.setSessionFromCookieString(cookieString)
         if self.session:
             self.session.updateLastVisited()
-            if moddebug and self.debug:
-                self.logger.debug('Session re-established: %s' % self.session)
         else:
             if moddebug and self.debug:
-                self.logger.debug('No session for cookieString: %s' % cookieString)
+                msg = 'No session for cookieString: %s' % cookieString
+                self.logger.debug(msg)
         
     def getRequestParam(self, paramName, default=None):
         params = self.getRequestParams()
@@ -419,6 +441,9 @@ class SessionView(ControlledAccessView):
 
     def getRequestParams(self):
         if self.requestParams == None:
+            if moddebug and self.debug:
+                msg = "Reading request params from request..."
+                self.logger.debug(msg)
             if self.request.POST:
                 self.requestParams = self.request.POST.copy()
             elif self.request.GET:
@@ -427,12 +452,14 @@ class SessionView(ControlledAccessView):
                 self.requestParams = self.getZeroRequestParams()
             #if self.request.FILES:
             #    self.requestParams.update(self.request.FILES)
-            if moddebug and self.debug:
-                self.logger.debug("Request params: %s" % self.requestParams)
+        if moddebug and self.debug:
+            msg = "Request params: %s" % self.requestParams
+            self.logger.debug(msg)
         return self.requestParams
 
     def getZeroRequestParams(self):
         return {}
+        #return MultiValueDict()
 
     def getResponse(self):
         if self.checkAccessControl():
@@ -445,7 +472,9 @@ class SessionView(ControlledAccessView):
         if not self.redirect:
             self.markNavigation()
         if not self.redirect:
-            self.setContext()
+            webkitName = self.dictionary[WEBKIT_NAME]
+            if webkitName == 'django':
+                self.setContext()
         self.createResponse()
         self.setSessionCookie()
         return self.response
@@ -469,7 +498,9 @@ class SessionView(ControlledAccessView):
         pass
 
     def createContext(self):
-        self.context = Context()
+        webkitName = self.dictionary[WEBKIT_NAME]
+        if webkitName == 'django':
+            self.context = Context()
     
     def markNavigation(self):
         self.markMajorNavigation()
@@ -518,14 +549,26 @@ class SessionView(ControlledAccessView):
                 item['isCurrentItem'] = False
 
     def createResponse(self):
+        webkitName = self.dictionary[WEBKIT_NAME]
         if self.redirect:
-            self.response = HttpResponseRedirect(self.redirect)
+            if webkitName == 'django':
+                self.response = HttpResponseRedirect(self.redirect)
         else:
             self.createContent()
-            self.response = HttpResponse(self.content)
+            if self.redirect:
+                if webkitName == 'django':
+                    self.response = HttpResponseRedirect(self.redirect)
+            else:
+                if webkitName == 'django':
+                    self.response = HttpResponse(self.content)
+                    self.response.status_code = self.responseStatusCode
+                elif webkitName == 'pylons':
+                    self.response = self.content
            
     def createContent(self):
-        self.content = self.renderContextWithTemplate()
+        webkitName = self.dictionary[WEBKIT_NAME]
+        if webkitName == 'django':
+            self.content = self.renderContextWithTemplate()
     
     def renderContextWithTemplate(self):
         template = self.getTemplate()
@@ -597,9 +640,11 @@ class SessionView(ControlledAccessView):
             systemMediaSocket = mediaHost
        
         pageCreated = dm.times.getLocalNowCWithZone()
+        localNow = dm.times.getLocalNow()
         
         self.context.update({
             'pageCreated'       : pageCreated,
+            'localNow'          : localNow,
             'view'              : self,
             'uriPrefix'         : self.dictionary[URI_PREFIX],
             'mediaPrefix'       : self.dictionary[MEDIA_PREFIX],
@@ -631,9 +676,10 @@ class SessionView(ControlledAccessView):
         })
 
     def setRedirect(self, redirectPath):
-        redirectPath = self.dictionary[URI_PREFIX] + redirectPath
-        self.logger.info('Redirecting to: %s' % redirectPath)
-        self.redirect = redirectPath
+        if redirectPath:
+            redirectPath = self.dictionary[URI_PREFIX] + redirectPath
+            self.logger.info('Redirecting to: %s' % redirectPath)
+            self.redirect = redirectPath
     
     def setRedirectLogin(self, returnPath=''):
         if not returnPath:
@@ -663,9 +709,17 @@ class SessionView(ControlledAccessView):
             maxAge = 315360000
         else:
             maxAge = None
-        self.response.set_cookie(
-            cookieName, cookieString, domain=cookieDomain, max_age=maxAge
-        )
+        if self.debug:
+            msg = 'Setting cookie %s: %s' % (cookieName, cookieString)
+            self.logger.debug(msg)
+        webkitName = self.dictionary[WEBKIT_NAME]
+        if webkitName == 'django':
+            self.response.set_cookie(
+                cookieName, cookieString, domain=cookieDomain, max_age=maxAge,
+                expires=maxAge
+            )
+        elif webkitName == 'pylons':
+            print "Todo: How to set the pylons cookie? Use AuthKit"
 
     def startSession(self, person):
         self.session = person.sessions.create()
@@ -680,6 +734,12 @@ class SessionView(ControlledAccessView):
         else:
             self.logger.info("Stopping session, but there was no session.")
 
+    def showInlineHelp(self):
+        return False
+
+    def hasListallPage(self):
+        return False
+
 
 class AbstractClassView(SessionView):
 
@@ -789,6 +849,7 @@ class AbstractFormView(AbstractInstanceView):
                 
     def getInitialParams(self):
         return {}
+        #return MultiValueDict()
 
     def isSubmissionRequest(self):
         requestParams = self.getRequestParams()
@@ -800,10 +861,10 @@ class AbstractFormView(AbstractInstanceView):
         if self.formErrors == None:
             if self.getRequestParams():
                 self.formErrors = self.getValidationErrors()
+                if moddebug and self.debug:
+                    self.logger.debug("Form errors: %s" % self.formErrors)
             else:
                 self.formErrors = self.getZeroFormErrors() 
-        if moddebug and self.debug:
-            self.logger.debug("Form errors: %s" % self.formErrors)
         return self.formErrors
 
     def getZeroFormErrors(self):
@@ -852,7 +913,10 @@ class AbstractFormView(AbstractInstanceView):
         self.setRedirect(self.makePostManipulateLocation())
         
     def makePostManipulateLocation(self):
-        raise Exception("Abstract method not implemented on class %s." % self.__class__.__name__)
+        msg = "Abstract method not implemented on class %s." % (
+            self.__class__.__name__
+        )
+        raise Exception(msg)
 
 
 class AbstractInputFormView(AbstractFormView):
@@ -870,19 +934,23 @@ class AbstractListView(AbstractClassView):
         super(AbstractListView, self).setContext()
         domainClassRegister = self.registry.getDomainClassRegister()
         objectRegister = self.getManipulatedObjectRegister()
-        objectRegisterCount = len(objectRegister)
-        isRegisterCountLow = objectRegisterCount < self.registerSizeThreshold
-        isRegisterCountSingle = objectRegisterCount == 1
-        isRegisterCountZero = objectRegisterCount == 0
+        objectCount = len(objectRegister)
+        isCountSingle = objectCount == 1
+        isCountZero = objectCount == 0
+        isCountLow = objectCount < self.registerSizeThreshold
         self.context.update({
-            'domainClassNameList' : domainClassRegister.keys(),  # todo: remove
-            'domainObjectList'    : objectRegister,
-            'domainClassName'     : objectRegister.typeName,
-            'objectRegister'      : objectRegister,
-            'registerCount'       : objectRegisterCount,
-            'isRegisterCountLow'  : isRegisterCountLow,
-            'isRegisterCountSingle' : isRegisterCountSingle,
-            'isRegisterCountZero' : isRegisterCountZero,
+            'domainClassNameList' : domainClassRegister.keys(), # todo: remove
+            'domainObjectList': objectRegister,
+            'domainClassName': objectRegister.typeName,
+            'objectRegister': objectRegister,
+            'objectCount': objectCount,
+            'isCountSingle': isCountSingle,
+            'isCountZero': isCountZero,
+            'isCountLow': isCountLow,
+            'showRegisterAllLink': not isCountLow,
+            'showRegisterTable': isCountLow,
+            'showRegisterIndex': not isCountLow,
+            'showRegisterSearch': False,
         })
 
 
@@ -908,17 +976,31 @@ class AbstractSearchView(AbstractClassView):
 
     def setContext(self):
         super(AbstractSearchView, self).setContext()
+        self.context.update({
+            'showRegisterSearch': True,
+            'showRegisterIndex': True,
+        })
         if self.startsWith or self.userQuery:
             searchResultList = self.searchManipulatedRegister()
+            objectCount = len(searchResultList)
             self.context.update({
-                'isResultsRequest' : True,
-                'domainObjectList' : searchResultList,
-                'resultsCount'     : len(searchResultList),
-                'isResultSingular' : len(searchResultList) == 1,
-                'userQuery'        : self.userQuery.replace('"', '&quot;'),
-                'startsWith'       : self.startsWith,
+                'isResultsRequest': True,
+                'domainObjectList': searchResultList,
+                'objectCount': objectCount,
+                'isCountZero': objectCount == 0,
+                'isCountSingle': objectCount == 1,
+                'userQuery': self.userQuery.replace('"', '&quot;'),
+                'startsWith': self.startsWith,
+                'showRegisterTable': bool(objectCount),
                 # todo: Improve above HTML character substitution.
             })
+        else:
+            objectRegister = self.getManipulatedObjectRegister()
+            objectCount = len(objectRegister)
+            self.context.update({
+                'objectCount': objectCount,
+                'showRegisterTable': False,
+            })
 
     def searchManipulatedRegister(self):
         if not self.domainClassName:
@@ -949,16 +1031,30 @@ class AbstractFindView(AbstractClassView):
 
     def setContext(self):
         super(AbstractFindView, self).setContext()
+        self.context.update({
+            'showRegisterSearch': True,
+            'showRegisterIndex': True,
+        })
         if self.startsWith:
             resultList = self.findDomainObjects()
+            objectCount = len(resultList)
             self.context.update({
-                'isResultsRequest' : True,
-                'domainObjectList' : resultList,
-                'resultsCount'     : len(resultList),
-                'isResultSingular' : len(resultList) == 1,
-                'startsWith'       : self.startsWith,
+                'isResultsRequest': True,
+                'domainObjectList': resultList,
+                'objectCount': objectCount,
+                'isCountZero': objectCount == 0,
+                'isCountSingle': objectCount == 1,
+                'startsWith': self.startsWith,
+                'showRegisterTable': bool(objectCount),
             })
-
+        else:
+            objectRegister = self.getManipulatedObjectRegister()
+            objectCount = len(objectRegister)
+            self.context.update({
+                'objectCount': objectCount,
+                'showRegisterTable': False,
+            })
+    
     def findDomainObjects(self):
         if not self.domainClassName:
             raise Exception("No domainClassName on '%s' set." % self)
@@ -1005,7 +1101,8 @@ class AbstractCreateView(AbstractInputFormView):
             self.setCreatedObject(manipulator.domainObject)
 
     def getCreateParams(self):
-        return self.getRequestParams()
+        createParams = self.getRequestParams()
+        return createParams
 
     def setCreatedObject(self, createdObject):
         self.domainObject = createdObject
@@ -1070,21 +1167,30 @@ class AbstractReadView(AbstractInstanceView):
     def getPreviousNextDomainObjects(self):
         domainObject = self.getManipulatedDomainObject()
         objectRegister = self.getManipulatedObjectRegister()
+        nextAscends = objectRegister.getDomainClass().nextAscends
         objectList = objectRegister.getSortedList()
-        nextObject = objectRegister.getNextObject(
-            objectList, domainObject
-        )
-        previousObject = objectRegister.getPreviousObject(
-            objectList, domainObject
-        )
+        if nextAscends:
+            nextObject = objectRegister.getNextObject(
+                objectList, domainObject
+            )
+            previousObject = objectRegister.getPreviousObject(
+                objectList, domainObject
+            )
+        else:
+            nextObject = objectRegister.getPreviousObject(
+                objectList, domainObject
+            )
+            previousObject = objectRegister.getNextObject(
+                objectList, domainObject
+            )
         return (previousObject, nextObject)
 
 
 class AbstractUpdateView(AbstractInputFormView):
 
     def getInitialParams(self):
-        domainObject = self.getManipulatedDomainObject()
-        return domainObject.asRequestParams()
+        manipulator = self.getManipulator()
+        return manipulator.getUpdateParams()
         
     def manipulateDomainObject(self):
         super(AbstractUpdateView, self).manipulateDomainObject()
