@@ -5,7 +5,8 @@ from dm.ioc import *
 from dm.exceptions import *
 from dm.strategy import MakeCookieString
 from dm.util.datastructure import MultiValueDict
-from dm.util.http import HttpRequest
+from dm.webkit import HttpRequest
+from dm.dictionarywords import WEBKIT_NAME
 
 def suite():
     suites = [
@@ -71,7 +72,9 @@ class ViewTestCase(TestCase):
     viewerName = None
     viewClass = None
     requestPath = ''
+    requiredViewContext = None
     requiredResponseClassName = 'HttpResponse'
+    requiredResponseContent = None
     requiredRedirect = ''
     requiredRedirectPath = ''
 
@@ -80,9 +83,9 @@ class ViewTestCase(TestCase):
         self.response = None
         self.viewSession = None
         self.initViewSession()
-        self.GET = {}
-        self.POST = {}
-        self.COOKIES = {}
+        self.GET = MultiValueDict()
+        self.POST = MultiValueDict()
+        self.COOKIES = MultiValueDict()
         self.initCookies()
         self.initPost()
         self.initGet()
@@ -115,9 +118,9 @@ class ViewTestCase(TestCase):
 
     def buildRequest(self):
         request = HttpRequest()
-        request.GET.update(self.GET)
-        request.POST.update(self.POST)
-        request.COOKIES.update(self.COOKIES)
+        request.GET = self.GET
+        request.POST = self.POST
+        request.COOKIES = self.COOKIES
         request.path = self.requestPath
         return request
 
@@ -142,16 +145,38 @@ class ViewTestCase(TestCase):
         self.failUnless(self.response, "No response?")
         self.failUnlessEqual(self.view.redirect, self.requiredRedirect)
         self.failUnlessEqual(self.view.redirectPath, self.requiredRedirectPath)
-        self.failUnlessEqual(
-            self.response.__class__.__name__,
-            self.requiredResponseClassName,
-            "%s != %s:\n\n%s" % (
+        if self.dictionary[WEBKIT_NAME] == 'django':
+            self.failUnlessEqual(
                 self.response.__class__.__name__,
                 self.requiredResponseClassName,
-                self.response,
-            ),
-        )
+                "%s != %s:\n\n%s" % (
+                    self.response.__class__.__name__,
+                    self.requiredResponseClassName,
+                    self.response,
+                ),
+            )
+            self.checkResponseContent()
+            self.checkViewContext()
 
+    def checkResponseContent(self):
+        self.failUnlessResponseContent()
+
+    def failUnlessResponseContent(self):
+        if self.requiredResponseContent != None:
+            requiredContent = self.requiredResponseContent
+            self.failUnlessEqual(self.response.content, requiredContent)
+
+    def checkViewContext(self):
+        self.failUnlessViewContext()
+
+    def failUnlessViewContext(self):
+        for (name, value) in self.getRequiredViewContext().items():
+            self.failUnless(self.view.context.has_key(name), "Name '%s' not in context: %s" % (name, self.view.context))
+            contextValue = self.view.context[name]
+            self.failUnlessEqual(value, contextValue)
+
+    def getRequiredViewContext(self):
+        return {}
 
 class TestSessionView(ViewTestCase):
 
@@ -312,3 +337,7 @@ class TestAbstractDeleteView(ViewTestCase):
     requiredRedirect = '/login/'
 
 
+class AdminSessionViewTestCase(ViewTestCase):
+
+    viewerName = 'admin'
+
