@@ -1,17 +1,22 @@
 from dm.ioc import *
 from dm.exceptions import KforgeCommandError
-from dm.util.manipulator import forms as djangoforms
-from dm.util.manipulator import validators as djangovalidators
-from dm.util.html import escape
+from dm.dictionarywords import WEBKIT_NAME
+from dm.webkit import ValidationError
+from dm.webkit import Manipulator
+from dm.webkit import htmlescape
 import re
+import dm.webkit
 
-class BaseManipulator(djangoforms.Manipulator):
+class BaseManipulator(Manipulator):
     "Supertype for domain model manipulators."
 
     registry   = RequiredFeature('DomainRegistry')
     dictionary = RequiredFeature('SystemDictionary')
     commands   = RequiredFeature('CommandSet')
     logger     = RequiredFeature('Logger')
+    webkit     = dm.webkit
+
+    sizeSelectMultiple = 4
 
     def getValidationErrors(self, data):
         try:
@@ -46,11 +51,11 @@ class BaseManipulator(djangoforms.Manipulator):
   
     def isTooLong(self, field_data, limit):
         if (len(field_data.strip()) > limit):
-            raise djangovalidators.ValidationError("This field is too long.")
+            raise ValidationError("This field is too long.")
   
     def isTooShort(self, field_data, limit):
         if (len(field_data.strip()) < limit):
-            raise djangovalidators.ValidationError("This field is too short.")
+            raise ValidationError("This field is too short.")
 
 
 class DomainObjectManipulator(BaseManipulator):
@@ -63,32 +68,32 @@ class DomainObjectManipulator(BaseManipulator):
         self.metaObject = self.objectRegister.getDomainClassMeta()
         self.domainObject = domainObject
         self.fieldNames = fieldNames
+        if self.fieldNames == None:
+            self.fieldNames = []
         self.fields = []
         msg = "Building manipulator fields for %s..." % (
             self.__class__.__name__
         )
         self.logger.debug(msg)
         self.buildFields()
-
-        self.fieldNames = []
-        for field in self.fields:
-            self.fieldNames.append(field.field_name)
-        msg = "Built %s manipulator with fields: %s" % (
-            self.__class__.__name__, self.fieldNames
+        if self.fieldNames == []:
+            for field in self.fields:
+                self.fieldNames.append(field.field_name)
+        msg = "Built '%s' manipulator with field names: %s" % (
+            self.__class__.__name__, ", ".join(self.fieldNames)
         )
         self.logger.debug(msg)
 
     def buildFields(self):
         if self.fieldNames:
-            msg = "Manipulator building field objects from fieldNames..."
+            msg = "Building manipulator field objects from fieldNames"
+            msg = "%s (%s)." % (msg, ", ".join(self.fieldNames))
             self.logger.debug(msg)
             for fieldName in self.fieldNames:
-                #for metaAttr in self.metaObject.attributes:
-                #    if metaAttr.name == fieldName:
-                #        self.buildField(metaAttr)
                 metaAttr = self.metaObject.attributeNames[fieldName]
                 self.buildField(metaAttr)
         else:
+            self.fieldNames = []
             msg = "Manipulator building fields from domain object meta..."
             self.logger.debug(msg)
             for metaAttr in self.metaObject.attributes:
@@ -111,92 +116,207 @@ class DomainObjectManipulator(BaseManipulator):
         field = None
         isFieldRequired = metaAttr.isRequired
         self.logger.debug("Building manipulator field: %s" % metaAttr.name)
+        webkitName = self.dictionary[WEBKIT_NAME]
+
         if metaAttr.isAssociateList:
             countChoices = metaAttr.countChoices(self.domainObject)
             choices = metaAttr.getAllChoices(self.domainObject)
-            field = djangoforms.SelectMultipleField(
-                field_name=metaAttr.name,
-                is_required=isFieldRequired,
-                choices=choices,
-                size=4,
-            )
-        elif metaAttr.isDomainObjectRef:
-            countChoices = metaAttr.countChoices(self.domainObject)
-            if True or countChoices <= 50:
-                choices = metaAttr.getAllChoices(self.domainObject)
-                choices = [('', '-- select option --')] + choices
-                field = djangoforms.SelectField(
+            if webkitName == 'django':
+                field = self.webkit.SelectMultipleField(
                     field_name=metaAttr.name,
                     is_required=isFieldRequired,
                     choices=choices,
+                    size=self.sizeSelectMultiple,
                 )
+            elif webkitName == 'pylons':
+                field = self.webkit.SelectField(
+                    metaAttr.name,
+                    multiple=True,
+                )
+        elif metaAttr.isDomainObjectRef:
+            countChoices = metaAttr.countChoices(self.domainObject)
+            # Todo: Revert this 'True', and introduce Ajax auto-completion
+            # instead of drop-down when more than count limit.
+            if True or countChoices <= 50:
+                choices = metaAttr.getAllChoices(self.domainObject)
+                choices = [('', '-- select option --')] + choices
+                if webkitName == 'django':
+                    field = self.webkit.SelectField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                        choices=choices,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.SelectField(
+                        metaAttr.name,
+                    )
             else:  # need to find a way to only do this for string keys
-                field = djangoforms.TextField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.TextField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
         elif metaAttr.isValueObject():
             if metaAttr.typeName == 'String':
                 if metaAttr.name[0:5] == 'email':
-                    field = djangoforms.EmailField(
-                        field_name=metaAttr.name,
-                        is_required=isFieldRequired,
-                        length=35
-                    )
+                    if webkitName == 'django':
+                        field = self.webkit.EmailField(
+                            field_name=metaAttr.name,
+                            is_required=isFieldRequired,
+                            length=35
+                        )
+                    elif webkitName == 'pylons':
+                        field = self.webkit.TextField(
+                            metaAttr.name,
+                            size=36
+                        )
                 else:
-                    field = djangoforms.TextField(
-                        field_name=metaAttr.name,
-                        is_required=isFieldRequired,
-                    )
+                    if webkitName == 'django':
+                        field = self.webkit.TextField(
+                            field_name=metaAttr.name,
+                            is_required=isFieldRequired,
+                        )
+                    elif webkitName == 'pylons':
+                        field = self.webkit.TextField(
+                            metaAttr.name,
+                            size=40
+                        )
                 
             elif metaAttr.typeName == 'Text':
-                field = djangoforms.LargeTextField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.LargeTextField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                        cols=30,
+                        rows=8,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
+            elif metaAttr.typeName == 'MarkdownText':
+                if webkitName == 'django':
+                    field = self.webkit.LargeTextField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                        cols=30,
+                        rows=8,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'Password':
-                field = djangoforms.PasswordField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.PasswordField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'Integer':
-                field = djangoforms.IntegerField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.IntegerField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'Boolean':
-                field = djangoforms.CheckboxField(
-                    field_name=metaAttr.name
-                    # doesn't accept 'is_required' parameter
-                )
+                if webkitName == 'django':
+                    field = self.webkit.CheckboxField(
+                        field_name=metaAttr.name
+                        # doesn't accept 'is_required' parameter
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'Url':
-                field = djangoforms.URLField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.URLField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'DateTime':
-                field = djangoforms.DatetimeField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.DatetimeField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             elif metaAttr.typeName == 'Date':
-                field = djangoforms.DateField(
-                    field_name=metaAttr.name,
-                    is_required=isFieldRequired,
-                )
+                if webkitName == 'django':
+                    field = self.webkit.DateField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
+            elif metaAttr.typeName == 'RDate':
+                if webkitName == 'django':
+                    field = self.webkit.RDateField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
             else:
-                field = djangoforms.TextField(
+                if webkitName == 'django':
+                    field = self.webkit.TextField(
+                        field_name=metaAttr.name,
+                        is_required=isFieldRequired,
+                    )
+                elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
+        elif metaAttr.typeName == 'ImageFile':
+            if webkitName == 'django':
+                field = self.webkit.FileUploadField(
                     field_name=metaAttr.name,
                     is_required=isFieldRequired,
                 )
-        elif metaAttr.typeName == 'ImageFile':
-            field = djangoforms.FileUploadField(
-                field_name=metaAttr.name,
-                is_required=isFieldRequired,
-            )
+            elif webkitName == 'pylons':
+                    field = self.webkit.TextField(
+                        metaAttr.name,
+                        size=40
+                    )
         if field:
-            field.metaAttr = metaAttr
-            field.field_comment = metaAttr.comment
+            if webkitName == 'django':
+                field.metaAttr = metaAttr
+                field.field_title = metaAttr.calcTitle()
+                field.field_comment = metaAttr.comment
             self.fields.append(field)
         else:
             message = "No form field for meta attribute: %s" % metaAttr
@@ -214,6 +334,7 @@ class DomainObjectManipulator(BaseManipulator):
         self.objectKwds = {}
         for metaAttr in self.metaObject.attributes:
             if not metaAttr.isAssociateList:
+                # Todo: Fix this up to check the fields like update().
                 if self.data.has_key(metaAttr.name):
                     attrValue = metaAttr.makeValueFromMultiValueDict(
                         self.data
@@ -256,14 +377,36 @@ class DomainObjectManipulator(BaseManipulator):
         self.setAssociateListAttributes()
 
     def setNonAssociateListAttributes(self):
+        self.setAttributesFromMultiValueDict(self.domainObject, self.data)
+        self.domainObject.save()
+       
+    def setAttributesFromMultiValueDict(self, domainObject, multiValueDict):
+        attrValues = {}
+        metaAttrs = {}
         for metaAttr in self.metaObject.attributes:
             if not metaAttr.isAssociateList:
                 if metaAttr.name in self.fieldNames:
-                    metaAttr.setAttributeFromMultiValueDict(
-                        self.domainObject, self.data
-                    )
-        self.domainObject.save()
-        
+                    if metaAttr.name in self.data:
+                        metaAttrs[metaAttr.name] = metaAttr
+        for attrName in metaAttrs:
+            metaAttr = metaAttrs[attrName]
+            try:
+                domValue = metaAttr.makeValueFromMultiValueDict(multiValueDict)
+                attrValues[attrName] = domValue
+            except Exception, inst:
+                msg = "Can't make '%s' value from multi-value dict: %s" % (
+                    attrName, multiValueDict
+                )
+                self.logger.error(msg)
+                raise
+        for attrName in metaAttrs:
+            metaAttr = metaAttrs[attrName]
+            attrValue = attrValues[attrName]
+            if attrValue != metaAttr.getAttributeValue(domainObject):
+                msg = "Setting '%s' attribute to '%s'." % (attrName, attrValue)
+                self.logger.debug(msg)
+                metaAttr.setAttributeValue(domainObject, attrValue)
+
     def setAssociateListAttributes(self):
         for metaAttr in self.metaObject.attributes:
             if metaAttr.isAssociateList:
@@ -282,6 +425,9 @@ class DomainObjectManipulator(BaseManipulator):
                 return field
         return None
 
+    def getUpdateParams(self):
+        return self.domainObject.asRequestParams()
+
 
 class HasManyManipulator(DomainObjectManipulator):
     "Domain object 'HasMany' attribute manipulator."
@@ -342,10 +488,14 @@ class FormWrapper(object):
         if not hasattr(self.manipulator, 'pluginFields'):
             return wrappedFields
         for pluginField in self.manipulator.pluginFields:
-            indicateRequired = ""
+            indication = ""
             if pluginField.is_required:
-                indicateRequired = "<strong>*</strong>"
-            wrappedField = "<label for=\"id_%s\">%s %s</label><br />" % (pluginField.field_name, pluginField.field_title, indicateRequired)
+                indication = "<strong>*</strong>"
+            wrappedField = "<label for=\"id_%s\">%s %s</label><br />" % (
+                pluginField.field_name,
+                pluginField.field_title,
+                indication
+            )
             wrappedField += "%s <br />" % self[pluginField.field_name]
             wrappedField += "<p class=\"desc\">%s</p>" % pluginField.field_comment
             wrappedFields.append(wrappedField)
@@ -367,6 +517,7 @@ class FormWrapper(object):
     def has_errors(self):
         return self.error_dict != {}
 
+
 class FormFieldWrapper(object):
     "A bridge between the template system and an individual form field. Used by FormWrapper."
     
@@ -381,7 +532,7 @@ class FormFieldWrapper(object):
 
     def makeErrorMessage(self):
         if len(self.error_list):
-            message =  escape(self.error_list[0])
+            message =  htmlescape(self.error_list[0])
             thisFieldPattern = re.compile('This field')
             confirmationPattern = re.compile('confirmation')
             underscorePattern = re.compile('_')
@@ -416,7 +567,7 @@ class FormFieldWrapper(object):
 
     def html_error_list(self):
         if self.errors():
-            return '<ul class="errorlist"><li>%s</li></ul>' % '</li><li>'.join([escape(e) for e in self.errors()])
+            return '<ul class="errorlist"><li>%s</li></ul>' % '</li><li>'.join([htmlescape(e) for e in self.errors()])
         else: 
             return ''
 
